class Base {

    constructor(name, parent = null, onInit = null) {
        /**
         * The name used to reference the object
         * Must be unique in a container object
         */
        if(! name || (name == undefined) )
        {
            throw new Error('Object constructor: Missig name parameter')
        }
        this.name = name

        if(parent != null)
        {
            if(!_.isObject(parent) )
            {
                throw new Error('The object parent is not an object.')
            }
            if(parent.hasOwnProperty(name))
            {
                throw new Error('A property with name [' + name + '] already exists in parent object.')
            }
        }

        /**
         * Reference the parent obiect
         */
        this.parent = parent      

        /**
         * Store informations about an object
         */
        this.comment = null

        /**
         * Any extra data needed
         */
        this.tag = null

        /** 
         * Occurs when an object is created 
         */
        this.onInit = onInit                                

        /**
         * Occurs when an object is released
         * Se pare ca acest eveniment nu exists
         * Il putem folosi custom
         */
        this.onDestroy = null

        /**
         * Occurs when there is a run-time error in a method
         * Nu stiu inca cum sa o apelez
         */
        this.onError = (message, method) => {
            CT.events.push(this.name + '::onError(' + message + ' (' + method + '))')
            throw new Error(message)
        }

        /**
         * Start
         */
        CT.events.push(this.name + '::construct()')
        if( _.isFunction(this.onInit) )
        {
            CT.events.push(this.name + '::onInit()')
            this.onInit(this)
        }
    }

    /** METHODS */

    /**
     * Check if an object as a property
     */
    PropertyExists(key) {
        return this.hasOwnProperty(key)
    }

    /**
     * Set a value to a property 
     */
    Set(key, value = null) {
        if( this.PropertyExists(key) ) 
        {
            if(Vue)
            {
                Vue.set(this, key, value)
            }
            else
            {
                this[key] = value
            }
            return this
        }
        throw new Error('Property [' + key+ '] not exists.')
    }

    /**
     * Get value of a property
     */
    GetPropertyValue(key) {
        if(this.PropertyExists(key))
        {
            if( _.isFunction(this[key]) )
            {
                /**
                 * Oare cu ce parametri sa se apeleze aici
                 */
                return this[key]()
            }
            return this[key]
        }
        throw new Error('Property [' + key+ '] not exists.')
    }

    /**
     * Add a new property to an object 
     */
    AddProperty(key, value = null) {
        if( ! this.PropertyExists(key) )
        {
            if(Vue)
            {
                Vue.set(this, key, value)
            }
            else
            {
                this[key] = value
            }
            return this
        }
        throw new Error('Property [' + key+ '] already exists in object [' + this.name + '].')
    }

    /**
     * Remove a property from an object 
     */
    RemoveProperty(key) {

    }

    /**
     * Set the object properties to default values
     */
    ResetToDefault() {

    }

}

export default Base